/**
 * Cloud functions to handle Firebase Performance Monitoring events from Firebase Alerts.
 * @packageDocumentation
 */
import { CloudEvent, CloudFunction } from '../../core';
import { EventHandlerOptions } from '../../options';
import { FirebaseAlertData } from './alerts';
/**
 * The internal payload object for a performance threshold alert.
 * Payload is wrapped inside a {@link FirebaseAlertData} object.
 */
export interface ThresholdAlertPayload {
    eventName: string;
    eventType: string;
    metricType: string;
    numSamples: number;
    thresholdValue: number;
    thresholdUnit: string;
    conditionPercentile?: number;
    appVersion?: string;
    violationValue: number;
    violationUnit: string;
    investigateUri: string;
}
/**
 * A custom CloudEvent for Firebase Alerts (with custom extension attributes).
 * @typeParam T - the data type for performance alerts that is wrapped in a `FirebaseAlertData` object.
 */
export interface PerformanceEvent<T> extends CloudEvent<FirebaseAlertData<T>> {
    /** The type of the alerts that got triggered. */
    alertType: string;
    /** The Firebase App ID that’s associated with the alert. */
    appId: string;
}
/**
 * Configuration for app distribution functions.
 */
export interface PerformanceOptions extends EventHandlerOptions {
    appId?: string;
}
/**
 * Declares a function that can handle receiving performance threshold alerts.
 * @param handler - Event handler which is run every time a threshold alert is received.
 * @returns A function that you can export and deploy.
 */
export declare function onThresholdAlertPublished(handler: (event: PerformanceEvent<ThresholdAlertPayload>) => any | Promise<any>): CloudFunction<PerformanceEvent<ThresholdAlertPayload>>;
/**
 * Declares a function that can handle receiving performance threshold alerts.
 * @param appId - A specific application the handler will trigger on.
 * @param handler - Event handler which is run every time a threshold alert is received.
 * @returns A function that you can export and deploy.
 */
export declare function onThresholdAlertPublished(appId: string, handler: (event: PerformanceEvent<ThresholdAlertPayload>) => any | Promise<any>): CloudFunction<PerformanceEvent<ThresholdAlertPayload>>;
/**
 * Declares a function that can handle receiving performance threshold alerts.
 * @param opts - Options that can be set on the function.
 * @param handler - Event handler which is run every time a threshold alert is received.
 * @returns A function that you can export and deploy.
 */
export declare function onThresholdAlertPublished(opts: PerformanceOptions, handler: (event: PerformanceEvent<ThresholdAlertPayload>) => any | Promise<any>): CloudFunction<PerformanceEvent<ThresholdAlertPayload>>;
